@@ -9,7 +9,7 @@ from typing import Any, Dict, Optional
 from urllib.parse import parse_qs, urlparse
 from tqdm.auto import tqdm
 
-from . import utils
+from . import utils, exceptions
 from .constants import FB_BASE_URL, FB_MOBILE_BASE_URL
 from .fb_types import Options, Post, RawPost, RequestFunction, Response, URL
 
@@ -178,6 +178,8 @@ class PostExtractor:
                     continue
 
                 post.update(partial_post)
+            except exceptions.TemporarilyBanned:
+                raise
             except Exception as ex:
                 log_warning("Exception while running %s: %r", method.__name__, ex)
 
@@ -954,6 +956,8 @@ class PostExtractor:
             except Exception as e:
                 logger.error(f"Unable to parse comment {comment}: {e}")
                 continue
+            inline_replies = comment.find("div[data-sigil='comment inline-reply']")
+            result["replies"] = [self.parse_comment(reply) for reply in inline_replies]
             replies = comment.find(
                 "div.async_elem[data-sigil='replies-see-more'] a[href],div[id*='comment_replies_more'] a[href]",
                 first=True,
@@ -975,7 +979,7 @@ class PostExtractor:
                     reply_selector = '#root div[id]'
                 replies = response.html.find(reply_selector)[1:]
                 try:
-                    result["replies"] = [self.parse_comment(reply) for reply in replies]
+                    result["replies"].extend([self.parse_comment(reply) for reply in replies])
                 except Exception as e:
                     logger.error(
                         f"Unable to parse comment {result['comment_id']} replies {replies}: {e}"
