@@ -6,12 +6,19 @@ import re
 from functools import partial
 from typing import Iterator, Union
 import json
+from urllib.parse import parse_qs, urlparse
 
 from requests import RequestException
 from requests_html import HTMLSession
 
 from . import utils
-from .constants import DEFAULT_PAGE_LIMIT, FB_BASE_URL, FB_MOBILE_BASE_URL, FB_W3_BASE_URL
+from .constants import (
+    DEFAULT_PAGE_LIMIT,
+    FB_BASE_URL,
+    FB_MOBILE_BASE_URL,
+    FB_W3_BASE_URL,
+    FB_MBASIC_BASE_URL,
+)
 from .extractors import extract_group_post, extract_post, extract_photo_post, PostExtractor
 from .fb_types import Post, Profile
 from .page_iterators import iter_group_pages, iter_pages, iter_photos
@@ -80,6 +87,9 @@ class FacebookScraper:
             post = {"original_request_url": post_url, "post_url": url}
             logger.debug(f"Requesting page from: {url}")
             response = self.get(url)
+            if "/watch/" in response.url:
+                video_id = parse_qs(urlparse(response.url).query).get("v")[0]
+                response = self.get(video_id)
             elem = response.html.find('[data-ft*="top_level_post_id"]', first=True)
             photo_post = False
             if response.html.find("div.msg", first=True):
@@ -128,40 +138,85 @@ class FacebookScraper:
                     post.pop('source', None)
             yield post
 
+    def get_friends(self, account, **kwargs) -> Iterator[Profile]:
+        friend_opt = kwargs.get("friends")
+        limit = None
+        if type(friend_opt) in [int, float]:
+            limit = friend_opt
+        friend_url = kwargs.pop("start_url", None)
+        if not friend_url:
+            friend_url = utils.urljoin(FB_MOBILE_BASE_URL, f'/{account}/friends/')
+        request_url_callback = kwargs.get('request_url_callback')
+        friends_found = 0
+        while friend_url:
+            logger.debug(f"Requesting page from: {friend_url}")
+            response = self.get(friend_url)
+            elems = response.html.find('div[data-sigil="undoable-action"]')
+            logger.debug(f"Found {len(elems)} friends")
+            for elem in elems:
+                name = elem.find("h3>a", first=True)
+                tagline = elem.find("div.notice.ellipsis", first=True).text
+                profile_picture = elem.find("i.profpic", first=True).attrs.get("style")
+                match = re.search(r"url\('(.+)'\)", profile_picture)
+                if match:
+                    profile_picture = utils.decode_css_url(match.groups()[0])
+                user_id = json.loads(
+                    elem.find("a.touchable[data-store]", first=True).attrs["data-store"]
+                ).get("id")
+                friend = {
+                    "id": user_id,
+                    "link": name.attrs.get("href"),
+                    "name": name.text,
+                    "profile_picture": profile_picture,
+                    "tagline": tagline,
+                }
+                yield friend
+                friends_found += 1
+            if limit and friends_found > limit:
+                return
+            more = re.search(r'href:"(/[^/]+/friends[^"]+)"', response.text)
+            if more:
+                friend_url = utils.urljoin(FB_MOBILE_BASE_URL, more.group(1))
+                if request_url_callback:
+                    request_url_callback(friend_url)
+            else:
+                return
+
     def get_profile(self, account, **kwargs) -> Profile:
         result = {}
 
-        logger.debug(f"Requesting page from: {account}")
-        response = self.get(account)
-        match = re.search(r'entity_id:(\d+),', response.html.html)
-        if match:
-            result["id"] = match.group(1)
-        photo_links = response.html.find("a[href^='/photo.php']")
-        if photo_links:
-            cover_photo = photo_links[0]
-            result["cover_photo_text"] = cover_photo.attrs.get("title")
-            response = self.get(cover_photo.attrs.get("href"))
-            extractor = PostExtractor(response.html, kwargs, self.get)
-            result["cover_photo"] = extractor.extract_photo_link_HQ(response.html.html)
+        if kwargs.get("allow_extra_requests", True):
+            logger.debug(f"Requesting page from: {account}")
+            response = self.get(account)
+            photo_links = response.html.find("a[href^='/photo.php']")
+            if photo_links:
+                cover_photo = photo_links[0]
+                result["cover_photo_text"] = cover_photo.attrs.get("title")
+                response = self.get(cover_photo.attrs.get("href"))
+                extractor = PostExtractor(response.html, kwargs, self.get)
+                result["cover_photo"] = extractor.extract_photo_link_HQ(response.html.html)
 
-            profile_photo = photo_links[1]
-            response = self.get(profile_photo.attrs.get("href"))
-            result["profile_picture"] = extractor.extract_photo_link_HQ(response.html.html)
-        else:
-            cover_photo = response.html.find(
-                "div[data-sigil='cover-photo']>i.img", first=True
-            )
-            if cover_photo:
-                match = re.search(r"url\('(.+)'\)", cover_photo.attrs["style"])
-                if match:
-                    result["cover_photo"] = utils.decode_css_url(match.groups()[0])
-            profpic = response.html.find("img.profpic", first=True)
-            if profpic:
-                result["profile_picture"] = profpic.attrs["src"]
+                profile_photo = photo_links[1]
+                response = self.get(profile_photo.attrs.get("href"))
+                result["profile_picture"] = extractor.extract_photo_link_HQ(response.html.html)
+            else:
+                cover_photo = response.html.find(
+                    "div[data-sigil='cover-photo']>i.img", first=True
+                )
+                if cover_photo:
+                    match = re.search(r"url\('(.+)'\)", cover_photo.attrs["style"])
+                    if match:
+                        result["cover_photo"] = utils.decode_css_url(match.groups()[0])
+                profpic = response.html.find("img.profpic", first=True)
+                if profpic:
+                    result["profile_picture"] = profpic.attrs["src"]
 
         about_url = utils.urljoin(FB_MOBILE_BASE_URL, f'/{account}/about/')
         logger.debug(f"Requesting page from: {about_url}")
         response = self.get(about_url)
+        match = re.search(r'entity_id:(\d+),', response.html.html)
+        if match:
+            result["id"] = match.group(1)
         # Profile name is in the title
         title = response.html.find("title", first=True).text
         if " | " in title:
@@ -218,50 +273,17 @@ class FacebookScraper:
                 ):  # Divisible by two, assume pairs
                     pairs = {}
                     for i in range(0, len(bits), 2):
-                        pairs[bits[i + 1]] = bits[i]
+                        if bits[i + 1] == "Websites":
+                            if "Websites" not in pairs:
+                                pairs["Websites"] = []
+                            pairs["Websites"].append(bits[i])
+                        else:
+                            pairs[bits[i + 1]] = bits[i]
                     result[header] = pairs
                 else:
                     result[header] = "\n".join(bits)
-        friend_opt = kwargs.get("friends")
-        if friend_opt:
-            limit = None
-            if type(friend_opt) in [int, float]:
-                limit = friend_opt
-            friend_url = utils.urljoin(FB_MOBILE_BASE_URL, f'/{account}/friends/')
-            elems = []
-            while friend_url:
-                logger.debug(f"Requesting page from: {friend_url}")
-                response = self.get(friend_url)
-                elems.extend(response.html.find('div[data-sigil="undoable-action"]'))
-                if limit and len(elems) > limit:
-                    break
-                more = re.search(r'href:"(/[^/]+/friends[^"]+)"', response.text)
-                if more:
-                    friend_url = utils.urljoin(FB_MOBILE_BASE_URL, more.group(1))
-                else:
-                    break
-            logger.debug(f"Found {len(elems)} friends")
-            friends = []
-            for elem in elems:
-                name = elem.find("h3>a", first=True)
-                tagline = elem.find("div.notice.ellipsis", first=True).text
-                profile_picture = elem.find("i.profpic", first=True).attrs.get("style")
-                match = re.search(r"url\('(.+)'\)", profile_picture)
-                if match:
-                    profile_picture = utils.decode_css_url(match.groups()[0])
-                user_id = json.loads(
-                    elem.find("a.touchable[data-store]", first=True).attrs["data-store"]
-                ).get("id")
-                friends.append(
-                    {
-                        "id": user_id,
-                        "link": name.attrs.get("href"),
-                        "name": name.text,
-                        "profile_picture": profile_picture,
-                        "tagline": tagline,
-                    }
-                )
-            result["Friends"] = friends
+        if kwargs.get("friends"):
+            result["Friends"] = list(self.get_friends(account, **kwargs))
         return result
 
     def get_page_info(self, page, **kwargs) -> Profile:
@@ -308,6 +330,9 @@ class FacebookScraper:
         return result
 
     def get_group_info(self, group, **kwargs) -> Profile:
+        self.set_user_agent(
+            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/603.3.8 (KHTML, like Gecko) Version/10.1.2 Safari/603.3.8"
+        )
         url = f'/groups/{group}'
         logger.debug(f"Requesting page from: {url}")
         resp = self.get(url).html
@@ -338,23 +363,51 @@ class FacebookScraper:
                 }
                 for e in admins
             ]
-            url = resp.find("a[href^='/browse/group/members']", first=True).attrs["href"]
-            members = []
-            while url:
-                logger.debug(f"Requesting page from: {url}")
-                resp = self.get(url).html
-                elems = resp.find("#root div.touchable a:not(.touchable)")
-                members.extend([{"name": e.text, "link": e.attrs["href"]} for e in elems])
-                more = re.search(r'"m_more_item",href:"([^"]+)', resp.text)
-                if more:
-                    url = more.group(1)
-                else:
-                    url = None
-            result["other_members"] = [m for m in members if m not in result["admins"]]
+            url = resp.find("a[href^='/browse/group/members']", first=True)
+            if url:
+                url = url.attrs["href"]
+                members = []
+                while url:
+                    logger.debug(f"Requesting page from: {url}")
+                    resp = self.get(url).html
+                    elems = resp.find("#root div.touchable a:not(.touchable)")
+                    members.extend([{"name": e.text, "link": e.attrs["href"]} for e in elems])
+                    more = re.search(r'"m_more_item",href:"([^"]+)', resp.text)
+                    if more:
+                        url = more.group(1)
+                    else:
+                        url = None
+                result["other_members"] = [m for m in members if m not in result["admins"]]
+            else:
+                logger.warning("No other members listed")
         except exceptions.LoginRequired as e:
             pass
         return result
 
+    def get_shop(self, page, **kwargs) -> Iterator[Post]:
+        self.set_user_agent(
+            "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/603.3.8 (KHTML, like Gecko) Version/10.1.2 Safari/603.3.8"
+        )
+        self.set_noscript(True)
+        url = f"{page}/shop/"
+        logger.debug(f"Fetching {url}")
+        resp = self.get(url)
+        more_links = resp.html.find("a[href]", containing="See More")
+        url = more_links[-1].attrs["href"]
+        logger.debug(f"Fetching {url}")
+        resp = self.get(url)
+        items = resp.html.find("div.be")
+        results = []
+        for item in items:
+            link_elem = item.find("div.bk.bl a", first=True)
+            name = link_elem.text
+            link = link_elem.attrs["href"]
+            image = item.find("img", first=True).attrs["src"]
+            price = item.find("div.bk.bl")[-1].text
+            result = {"name": name, "link": link, "image": image, "price": price}
+            results.append(result)
+        return results
+
     def get_group_posts(self, group: Union[str, int], **kwargs) -> Iterator[Post]:
         iter_pages_fn = partial(iter_group_pages, group=group, request_fn=self.get, **kwargs)
         return self._generic_get_posts(extract_group_post, iter_pages_fn, **kwargs)
@@ -381,7 +434,7 @@ class FacebookScraper:
                 response = self.submit_form(response)
             if (
                 response.url.startswith(FB_MOBILE_BASE_URL)
-                and "noscript" not in response.html.html
+                and not response.html.find("script", first=True)
                 and self.session.cookies.get("noscript") != "1"
             ):
                 warnings.warn(
@@ -397,6 +450,8 @@ class FacebookScraper:
             if title:
                 if title.text.lower() in not_found_titles:
                     raise exceptions.NotFound(title.text)
+                elif title.text.lower() == "error":
+                    raise exceptions.UnexpectedResponse("Your request couldn't be processed")
                 elif title.text.lower() in temp_ban_titles:
                     raise exceptions.TemporarilyBanned(title.text)
                 elif ">Your Account Has Been Disabled<" in response.html.html:
